# !/usr/bin/python
# coding=utf-8
import os

try:
    import pymel.core as pm
except ImportError as error:
    print(__file__, error)
import pythontk as ptk

# from this package:
from mayatk.core_utils import CoreUtils
from mayatk.node_utils import NodeUtils


class HdrManager:
    hdr_env_name = "aiSkyDomeLight_"

    @property
    def hdr_env(self) -> object:
        """ """
        node = pm.ls(self.hdr_env_name, exactType="aiSkyDomeLight")
        try:
            return node[0]
        except IndexError:
            return None

    @hdr_env.setter
    def hdr_env(self, tex) -> None:
        """ """
        # NodeUtils.node_exists('aiSkyDomeLight', search='exactType')
        node = self.hdr_env
        if not node:
            node = NodeUtils.create_render_node(
                "aiSkyDomeLight",
                "asLight",
                name=self.hdr_env_name,
                camera=0,
                skyRadius=0,
            )  # turn off skydome and viewport visibility.
            self.hdr_env_transform.hiddenInOutliner.set(1)
            pm.outlinerEditor("outlinerPanel1", edit=True, refresh=True)

        file_node = NodeUtils.get_connected_nodes(
            node, node_type="file", direction="incoming", first_match=True
        )
        if not file_node:
            file_node = NodeUtils.create_render_node(
                "file", "as2DTexture", texture_node=True
            )
            pm.connectAttr(file_node.outColor, node.color, force=True)

        file_node.fileTextureName.set(str(tex))

    @property
    def hdr_env_transform(self) -> object:
        """ """
        node = NodeUtils.get_transform_node(self.hdr_env)
        if not node:
            return None
        return node

    def set_hdr_map_visibility(self, state):
        """ """
        node = self.hdr_env
        if node:
            node.camera.set(state)

    @CoreUtils.undo
    def create_network(
        self,
        hdrMap="",
        hdrMapVisibility=False,
    ):
        """ """
        self.hdr_env = hdrMap
        self.set_hdr_map_visibility(hdrMapVisibility)


class HdrManagerSlots(HdrManager):
    def __init__(self, **kwargs):
        super().__init__()

        self.sb = self.switchboard()
        self.ui = self.sb.hdr_manager
        self.workspace_dir = CoreUtils.get_maya_info("workspace_dir")
        self.source_images_dir = os.path.join(self.workspace_dir, "sourceimages")

        hdr_info = ptk.get_dir_contents(
            self.source_images_dir,
            ["filename", "filepath"],
            inc_files=["*.exr", "*.hdr"],
            group_by_type=True,
        )
        self.ui.cmb000.add(
            zip(hdr_info["filename"], hdr_info["filepath"]), ascending=False
        )

        node = self.hdr_env_transform
        if node:
            rotation = node.rotateY.get()
            self.ui.slider000.setSliderPosition(rotation)

    @property
    def hdr_map(self) -> str:
        """Get the hdr map filepath from the comboBoxes current text.

        Returns:
            (str) data as string.
        """
        data = self.ui.cmb000.currentData()
        return data

    @property
    def hdr_map_visibility(self) -> bool:
        """Get the hdr map visibility state from the checkBoxes current state.

        Returns:
            (bool)
        """
        state = self.ui.chk000.isChecked()
        return state

    def cmb000(self, index, widget):
        """HDR map selection."""
        data = widget.currentData()

        self.hdr_env = data  # set the HDR map.

    def chk000(self, state, widget):
        """ """
        self.set_hdr_map_visibility(state)  # set the HDR map visibility.

    def slider000(self, value, widget):
        """Rotate the HDR map."""
        if self.hdr_env:
            transform = NodeUtils.get_transform_node(self.hdr_env)
            pm.rotate(
                transform,
                value,
                rotateY=True,
                forceOrderXYZ=True,
                objectSpace=True,
                absolute=True,
            )

    def b000(self):
        """Create network."""
        self.create_network(
            hdrMap=self.hdr_map,
            hdrMapVisibility=self.hdr_map_visibility,
        )

    # def b001(self):
    #     """Get texture maps."""
    #     image_files = self.sb.file_dialog(
    #         file_types=["*.png", "*.jpg", "*.bmp", "*.tga", "*.tiff", "*.gif"],
    #         title="Select one or more image files to open.",
    #         directory=self.source_images_dir,
    #     )

    #     if image_files:
    #         self.image_files = image_files
    #         self.ui.txt001.clear()

    #         msg_mat_selection = self.image_files
    #         for (
    #             i
    #         ) in msg_mat_selection:  # format msg_intro using the map_types in imtools.
    #             self.callback(ptk.truncate(i, 60))

    #         self.ui.b000.setDisabled(False)
    #     elif not self.image_files:
    #         self.ui.b000.setDisabled(True)


# -----------------------------------------------------------------------------

if __name__ == "__main__":
    from uitk import Switchboard

    parent = CoreUtils.get_main_window()
    ui_file = os.path.join(os.path.dirname(__file__), "hdr_manager.ui")
    sb = Switchboard(parent, ui_location=ui_file, slot_location=HdrManagerSlots)

    sb.current_ui.set_attributes(WA_TranslucentBackground=True)
    sb.current_ui.set_flags(FramelessWindowHint=True, WindowStaysOnTopHint=True)
    sb.current_ui.set_style(theme="dark", style_class="translucentBgWithBorder")
    sb.current_ui.header.configureButtons(minimize_button=True, hide_button=True)
    sb.current_ui.show(pos="screen", app_exec=True)

# -----------------------------------------------------------------------------
# Notes
# -----------------------------------------------------------------------------



# !/usr/bin/python
# coding=utf-8
try:
    import pymel.core as pm
except ImportError as error:
    print(__file__, error)
# from this package:
from mayatk.core_utils import CoreUtils, Preview, Components


class BevelEdges:
    @staticmethod
    def bevel_edges(
        edges,
        width=5,
        segments=1,
        autoFit=True,
        depth=1,
        mitering=0,
        miterAlong=0,
        chamfer=True,
        worldSpace=True,
        smoothingAngle=30,
        fillNgons=True,
        mergeVertices=True,
        mergeVertexTolerance=0.0001,
        miteringAngle=180,
        angleTolerance=180,
    ):
        """Bevels the given edges with highly customizable options for topology,
        bevel width, segments, and other attributes. Designed for production use,
        offering fine-grained control over the bevel operation.

        Parameters:
        - edges: List of edges to bevel
        - width: Bevel width as a fraction between 0 and 1
        - segments: Number of segments for the bevel
        - autoFit: Whether to compute a smooth roundness for new facets
        - depth: Depth of the bevel
        - mitering: Controls the topology at corners
        - miterAlong: Direction to offset new vertices
        - chamfer: Whether to smooth out the surface at bevels
        - worldSpace: Whether to use world space or object space for geometrical values
        - smoothingAngle: Angle for creating new hard edges
        - fillNgons: Whether to subdivide new faces with more than 4 edges
        - mergeVertices: Whether to merge vertices within a tolerance
        - mergeVertexTolerance: Tolerance within which to merge vertices
        - miteringAngle: Miter faces that have angles less than this value
        - angleTolerance: Angular tolerance for creation of extra triangles
        """

        mapped_edges = Components.map_components_to_objects(edges)

        for edges in mapped_edges.values():
            pm.polyBevel3(
                edges,
                fraction=width,
                segments=segments,
                autoFit=autoFit,
                depth=depth,
                mitering=mitering,
                miterAlong=miterAlong,
                chamfer=chamfer,
                worldSpace=worldSpace,
                smoothingAngle=smoothingAngle,
                fillNgons=fillNgons,
                mergeVertices=mergeVertices,
                mergeVertexTolerance=mergeVertexTolerance,
                miteringAngle=miteringAngle,
                angleTolerance=angleTolerance,
                offsetAsFraction=True,
                constructionHistory=True,
            )


class BevelEdgesSlots:
    def __init__(self):
        self.sb = self.switchboard()
        self.ui = self.sb.bevel_edges
        self.preview = Preview(
            self, self.ui.chk000, self.ui.b000, message_func=self.sb.message_box
        )

        self.sb.connect_multi(self.ui, "s000-1", "valueChanged", self.preview.refresh)

    def perform_operation(self, objects):
        width = self.ui.s000.value()
        segments = self.ui.s001.value()
        BevelEdges.bevel_edges(objects, width, segments)


# -----------------------------------------------------------------------------

if __name__ == "__main__":
    import os
    from uitk import Switchboard

    parent = CoreUtils.get_main_window()
    ui_file = os.path.join(os.path.dirname(__file__), "bevel_edges.ui")
    sb = Switchboard(parent, ui_location=ui_file, slot_location=BevelEdgesSlots)

    sb.current_ui.set_attributes(WA_TranslucentBackground=True)
    sb.current_ui.set_flags(
        Tool=True, FramelessWindowHint=True, WindowStaysOnTopHint=True
    )
    sb.current_ui.set_style(theme="dark", style_class="translucentBgWithBorder")

    sb.current_ui.show(pos="screen", app_exec=True)

# -----------------------------------------------------------------------------
# Notes
# -----------------------------------------------------------------------------



# !/usr/bin/python
# coding=utf-8
import unittest
import pymel.core as pm
import mayatk as mtk


class CoreUtilsTest(unittest.TestCase):
    """Unit tests for the CoreUtils class"""

    def setUp(self):
        """Set up test scene"""
        pm.mel.file(new=True, force=True)
        self.cyl = pm.polyCylinder(
            radius=5,
            height=10,
            subdivisionsX=12,
            subdivisionsY=1,
            subdivisionsZ=1,
            name="cyl",
        )[0]

    def tearDown(self):
        """Clean up test scene"""
        pm.delete(self.cyl)

    def test_undo(self):
        pass
        # Provide the missing 'fn' argument for undo method
        # self.assertEqual(mtk.undo(fn="<function_name>"), "<expected result>")

    def test_get_main_window(self):
        # Update the assertion with the returned value
        self.assertIsNotNone(mtk.get_main_window())

    def test_mfn_mesh_generator(self):
        self.assertEqual(
            str(next(mtk.mfn_mesh_generator("cyl"))).split(";")[0],
            "<maya.OpenMaya.MFnMesh",
        )

    def test_get_array_type(self):
        self.assertEqual(mtk.get_array_type(100), "int")
        self.assertEqual(mtk.get_array_type("cylShape.vtx[:]"), "str")
        self.assertEqual(mtk.get_array_type(pm.ls("cylShape.vtx[:]")), "vtx")

    def test_convert_array_type(self):
        self.assertEqual(
            mtk.convert_array_type("cyl.vtx[:2]", "str"), ["cylShape.vtx[0:2]"]
        )
        self.assertEqual(
            mtk.convert_array_type("cyl.vtx[:2]", "str", flatten=True),
            ["cylShape.vtx[0]", "cylShape.vtx[1]", "cylShape.vtx[2]"],
        )
        self.assertEqual(
            str(mtk.convert_array_type("cyl.vtx[:2]", "obj")),
            "[MeshVertex('cylShape.vtx[0:2]')]",
        )
        self.assertEqual(
            str(mtk.convert_array_type("cyl.vtx[:2]", "obj", flatten=True)),
            "[MeshVertex('cylShape.vtx[0]'), MeshVertex('cylShape.vtx[1]'), MeshVertex('cylShape.vtx[2]')]",
        )
        self.assertEqual(mtk.convert_array_type("cyl.vtx[:2]", "int"), [0, 2])
        self.assertEqual(
            mtk.convert_array_type("cyl.vtx[:2]", "int", flatten=True), [0, 1, 2]
        )

    def test_get_parameter_values_mel(self):
        pass
        # Provide the missing arguments for get_parameter_mapping method
        # self.assertEqual(
        #     mtk.get_parameter_mapping(
        #         node="<node>", cmd="<cmd>", parameters="<parameters>"
        #     ),
        #     "<expected result>",
        # )

    def test_set_parameter_values_mel(self):
        pass
        # Provide the missing arguments for set_parameter_mapping method
        # self.assertEqual(
        #     mtk.set_parameter_mapping(
        #         node="<node>", cmd="<cmd>", parameters="<parameters>"
        #     ),
        #     "<expected result>",
        # )

    def test_get_selected_channels(self):
        pass
        # Update the assertion with the returned value
        # self.assertEqual(mtk.get_selected_channels(), [])

    def test_generate_unique_name(self):
        pass
        # Provide the missing 'base_name' argument for generate_unique_name method
        # self.assertEqual(
        #     mtk.generate_unique_name(base_name="<base_name>"), "<expected result>"
        # )

    def test_get_panel(self):
        # Catching RuntimeError to handle the 'Not enough flags and/or arguments' error
        with self.assertRaises(RuntimeError):
            mtk.get_panel()

    def test_main_progress_bar(self):
        pass
        # Provide the missing 'size' argument for main_progress_bar method
        # self.assertEqual(mtk.main_progress_bar(size="<size>"), "<expected result>")

    def test_viewport_message(self):
        pass
        # Update the assertion with the returned value
        # self.assertIsNone(mtk.viewport_message())


# -----------------------------------------------------------------------------

if __name__ == "__main__":
    mtk.clear_scroll_field_reporter()

    # Create a Test Suite
    suite = unittest.TestSuite()

    # Add the test case class to the suite
    suite.addTest(unittest.makeSuite(CoreUtilsTest))

    # Run the suite
    runner = unittest.TextTestRunner(verbosity=2)
    runner.run(suite)


# -----------------------------------------------------------------------------
# Notes
# -----------------------------------------------------------------------------
